@@ -24,27 +24,12 @@ let _ = Random.self_init ()
 
 let debug = false
 
-let tempdir prefix suffix f =
-  let rec create_dir () =
-    try
-      let dirname = Filename.concat Filename.temp_dir_name (Printf.sprintf "%s%i%s" prefix (Random.int 10000) suffix) in 
-      Unix.mkdir dirname  0o700; dirname
-    with | Unix.Unix_error (Unix.EEXIST, _, _) -> create_dir () in
-  let dirname = create_dir () in
-  let workdir_bak = Sys.getcwd () in
-  Sys.chdir dirname;
-  let res = f dirname in
-  Sys.chdir workdir_bak;
-  Array.iter (fun x -> Sys.remove (Filename.concat dirname x)) (Sys.readdir dirname);
-  Unix.rmdir dirname;
-  res
-  
+let tempdir = Metapost_tool.tempdir
 
 type t = {tex : Dev_save.t;
           trans : Matrix.t}
 
 module Saved_device = Dviinterp.Interp(Dev_save.Dev_save)
-module Cairo_device = Dev_save.Dev_load(Dvicairo.Cairo_device)
 
 let set_verbosity b = 
   Saved_device.set_verbosity b
@@ -70,16 +55,19 @@ let create prelude = function
 (fun fmt -> List.iter (Printf.fprintf fmt
 "\\mpxshipout
 %s\\stopmpxshipout")) texs in
-  let todo pwd = 
+  let todo _ pwd = 
     let latex = genfile_name^".tex" in
     let file = open_out latex in
     Printf.fprintf file "%t" format;
     close_out file;
-    let exit_status = Sys.command (sprintf "%s -halt-on-error %s > gentex_dev_null.log" com_latex latex) in
-    if exit_status <> 0 then failwith (sprintf "Error with : %s : %s %s log in gentex.log" pwd com_latex latex);
+    let exit_status = Sys.command (sprintf "%s -halt-on-error %s > \
+gentex_dev_null.log" com_latex latex) in
+    if exit_status <> 0 then failwith (sprintf "Error with : %s : \
+%s %s log in gentex.log" pwd com_latex latex);
     let dvi = genfile_name^".dvi" in
     let saved = Saved_device.load_file true dvi in
-    List.map (fun x -> {tex = x;trans= Matrix.identity}) (Dev_save.separe_pages saved) in
+    List.map (fun x -> {tex = x;trans= Matrix.identity}) 
+      (Dev_save.separe_pages saved) in
   tempdir genfile_name "" todo
       
 let point_of_cm cm = (0.3937 *. 72.) *. cm
@@ -106,11 +94,3 @@ let print fmt tex =
   let min,max = bounding_box tex in
   Format.fprintf fmt "[%a,%a]" print min print max
 
-let draw cr tex = 
-  Cairo.save cr;
-  Cairo.transform cr (Matrix.to_cairo tex.trans);
-  Cairo_device.replay false tex.tex 
-  {Dvicairo.pic = cr;new_page = (fun () -> assert false);
-   x_origin = 0.; y_origin = 0.};
-  Cairo.restore cr
-  (*;Format.printf "Gentex : %a@." print tex*)
