@@ -16,8 +16,8 @@
 
 val set_verbosity : bool -> unit
 
-type t
-val draw : Cairo.t -> t -> unit
+type t = {tex : Dev_save.t;
+          trans : Matrix.t}
 val create : string -> string list -> t list
 val get_dimen_pt : t -> float * float * float * float
 val get_dimen_cm : t -> float * float * float * float
