@@ -112,18 +112,21 @@ and transform acc t =
     | TRScaled f | TRSlanted f | TRXscaled f | TRYscaled f -> num acc f
     | TRShifted p | TRZscaled p | TRRotateAround (p,_)-> point acc p
     | TRReflect (p1,p2) -> point (point acc p1) p2
+    | TRMatrix p -> 
+        num (num (num (num (num (num acc p.x0) p.y0) p.xx) p.xy) p.yx) p.yy
 and dash acc d =
   match d.Hashcons.node with
     | DEvenly | DWithdots -> acc
-    | DScaled (_,d) -> dash acc d
+    | DScaled (n,d) -> dash (num acc n) d
     | DShifted (p,d) -> point (dash acc d) p
     | DPattern l -> List.fold_left dash_pattern acc l
 and dash_pattern acc o =
   match o.Hashcons.node with
     | On f | Off f -> num acc f
 and command' acc = function
-  | CDraw (p, _, pe, dsh) 
-      -> path ((option_compile pen) ((option_compile dash) acc dsh) pe) p
+  | CDraw (p, b) -> 
+      let {color = _; pen = pe; dash = dsh} = b.Hashcons.node in
+      path ((option_compile pen) ((option_compile dash) acc dsh) pe) p
   | CFill (p,_) -> path acc p
   | CDotLabel (pic,_,pt) | CLabel (pic,_,pt) -> commandpic (point acc pt) pic
   | CExternalImage _ -> acc
@@ -159,13 +162,26 @@ let pathl arg = ct_auxl path arg; List.map Compute.path arg
 let metapathl arg = ct_auxl metapath arg; List.map Compute.metapath arg
 let picturel arg = ct_auxl picture arg; List.map Compute.picture arg
 
-let commandl_error ferror arg = ct_auxl command arg; List.map (ferror Compute.command) arg
-let commandpicl_error ferror arg = ct_auxl commandpic arg; List.map (ferror Compute.commandpic) arg
-let numl_error ferror arg = ct_auxl num arg; List.map (ferror Compute.num) arg
-let pointl_error ferror  arg = ct_auxl point arg; List.map (ferror Compute.point) arg
-let pathl_error ferror  arg = ct_auxl path arg; List.map (ferror Compute.path) arg
-let metapathl_error ferror  arg = ct_auxl metapath arg; List.map (ferror Compute.metapath) arg
-let picturel_error ferror arg = ct_auxl picture arg; List.map (ferror Compute.picture) arg
+let commandl_error ferror arg = ct_auxl command arg; 
+  List.map (ferror Compute.command) arg
+let commandpicl_error ferror arg = ct_auxl commandpic arg; 
+  List.map (ferror Compute.commandpic) arg
+let numl_error ferror arg = ct_auxl num arg; 
+  List.map (ferror Compute.num) arg
+let pointl_error ferror  arg = ct_auxl point arg; 
+  List.map (ferror Compute.point) arg
+let pathl_error ferror  arg = ct_auxl path arg; 
+  List.map (ferror Compute.path) arg
+let metapathl_error ferror  arg = ct_auxl metapath arg; 
+  List.map (ferror Compute.metapath) arg
+let picturel_error ferror arg = ct_auxl picture arg; 
+  List.map (ferror Compute.picture) arg
+
+
+
+let compute_nums () =
+  let l = ref [] in
+  (fun n -> l:= num !l n),(fun () -> compile_tex !l;l:=[])
 
 
 let commandpic arg = ct_aux commandpic arg; Compute.commandpic arg
@@ -175,3 +191,6 @@ let point arg = ct_aux point arg; Compute.point arg
 let path arg = ct_aux path arg; Compute.path arg
 let metapath arg = ct_aux metapath arg; Compute.metapath arg
 let picture arg = ct_aux picture arg; Compute.picture arg
+let transform arg = ct_auxl transform arg; 
+  List.fold_left (fun acc t -> Matrix.multiply acc (Compute.transform t)) 
+    Matrix.identity arg
