@@ -14,31 +14,89 @@
 (*                                                                        *)
 (**************************************************************************)
 
-open Types
+open Concrete_types
 
-type st = Types.scolor = 
+type st = Concrete_types.scolor =
   | RGB of float * float * float
   | CMYK of float * float * float * float
   | Gray of float
 
-type t = Types.color = 
+type t = Concrete_types.color =
   |OPAQUE of scolor
   |TRANSPARENT of float * scolor
 
+let rgb8 r g b =
+  OPAQUE (RGB ((float r)/.255., (float g)/.255., (float b)/.255.))
+let rgb8a r g b a =
+  TRANSPARENT ((float a)/.255.,
+               RGB ((float r)/.255., (float g)/.255., (float b)/.255.))
+let rgb_from_int i =
+  let b = i land 0xFF in
+  let g = (i land 0xFF00) lsr 8 in
+  let r = (i land 0xFF0000) lsr 16 in
+  rgb8 r g b
+
+    (* http://en.wikipedia.org/wiki/HSL_and_HSV *)
+let hsv h s v =
+  assert (0.<= s && s<=1.);
+  assert (0.<= v && v<=1.);
+  let c  = v *. s in
+  let h' = h /. 60. in
+  let x = c *. (1. -. (abs_float ((mod_float h' 2.) -. 1.))) in
+  let (r_1, g_1, b_1) =
+    if      0. <= h' && h' < 1. then (c,x,0.)
+    else if 1. <= h' && h' < 2. then (x,c,0.)
+    else if 2. <= h' && h' < 3. then (0.,c,x)
+    else if 3. <= h' && h' < 4. then (0.,x,c)
+    else if 4. <= h' && h' < 5. then (x,0.,c)
+    else if 5. <= h' && h' < 6. then (c,0.,x)
+    else (0.,0.,0.) in
+  let m = v -. c in
+  OPAQUE (RGB (r_1 +. m, g_1 +. m, b_1 +. m))
+
+let color_gen s v =
+  let choices = ref [] in
+  let value = 180. in
+  let next = ref 0. in
+  fun () ->
+    let rec aux acc value current = function 
+      | [] -> 
+          assert (current = 0.);
+          next := value;
+          List.rev_append acc [true]
+      | true::l ->
+          aux (false::acc) (value/.2.) (current-.value) l
+      | false::l -> 
+          next := current +. value;
+          List.rev_append acc (true::l) in
+    let res = !next in
+    choices := aux [] value res !choices;
+    hsv res s v
+
+
 let red = OPAQUE (RGB (1.0,0.0,0.0))
 let lightred = OPAQUE (RGB (1.0,0.5,0.5))
 let blue = OPAQUE (RGB (0.0,0.0,1.0))
-let lightblue = OPAQUE (RGB (0.68,0.85,0.90))
-let green = OPAQUE (RGB (0.0,1.0,0.0))
-let lightgreen = OPAQUE (RGB (0.5,1.0,0.5))
-let orange = OPAQUE (RGB (1.0,0.4,0.0))
-let purple = OPAQUE (RGB (0.6,0.0,0.6))
+let lightblue = rgb_from_int 0xADD8E6
+let green = rgb_from_int 0x008000
+let lightgreen = rgb_from_int 0x90EE90
+let orange = rgb_from_int 0xFFA500
+let purple = rgb_from_int 0x7F007F
 let magenta = OPAQUE (RGB (1.0,0.0,1.0))
-let lightmagenta = OPAQUE (RGB (1.0,0.5,1.0))
 let cyan =  OPAQUE (RGB (0.0,1.0,1.0))
-let lightcyan =  OPAQUE (RGB (0.5,1.0,1.0))
+let lightcyan = rgb_from_int 0xE0FFFF
 let yellow = OPAQUE (RGB (1.0,1.0,0.0))
-let lightyellow = OPAQUE (RGB (1.0,1.0,0.5))
+let lightyellow = rgb_from_int 0xFFFFE0
+  (* these colors do not correspond to neither X11 nor HTML colors
+     - commented out*)
+(* let lightblue = OPAQUE (RGB (0.5,0.5,1.0)) *)
+(* let green = OPAQUE (RGB (0.0,1.0,0.0)) *)
+(* let lightgreen = OPAQUE (RGB (0.5,1.0,0.5)) *)
+(* let orange = OPAQUE (RGB (1.0,0.4,0.0)) *)
+(* let purple = OPAQUE (RGB (0.6,0.0,0.6)) *)
+(* let lightcyan =  OPAQUE (RGB (0.5,1.0,1.0)) *)
+(* let lightyellow = OPAQUE (RGB (1.0,1.0,0.5)) *)
+let lightmagenta = OPAQUE (RGB (1.0,0.5,1.0))
 let gray f = OPAQUE (Gray f)
 let white = gray 1.0
 let lightgray = gray 0.75
@@ -53,15 +111,12 @@ let cmyka c m y k a = TRANSPARENT (a,CMYK (c,m,y,k))
 
 let is_opaque = function |OPAQUE _ -> true | _ -> false
 let opaque = function |TRANSPARENT (_,c) -> OPAQUE c | c -> c
-let transparent f = function 
-  |TRANSPARENT (f2,c) -> TRANSPARENT (f*.f2,c) 
+let transparent f = function
+  |TRANSPARENT (f2,c) -> TRANSPARENT (f*.f2,c)
   |OPAQUE c -> TRANSPARENT (f,c)
 
-
 let colors : (string, t) Hashtbl.t = Hashtbl.create 91
 let color n = Hashtbl.find colors n
-let rgb8 r g b = OPAQUE (RGB ((float r)/.255., (float g)/.255., (float b)/.255.))
-let rgb8a r g b a = TRANSPARENT ((float a)/.255.,RGB ((float r)/.255., (float g)/.255., (float b)/.255.))
 (** generated part *)
 let _ = Hashtbl.add colors "snow" (rgb8 255 250 250)
 let _ = Hashtbl.add colors "ghost" (rgb8 248 248 255)
