@@ -232,7 +232,7 @@ and path' = function
       let p', code = path_save p in
       begin
         match p' with
-        | C.PAName x -> C.PASub (f1,f2,x), c1 ++ c2
+        | C.PAName x -> C.PASub (f1,f2,x), c1 ++ c2 ++ code
         | _ -> assert false
       end
   | PABBox p ->
@@ -332,6 +332,22 @@ and transform t =
   | TRRotateAround (p,f) ->
       let p, code = point p in
         C.TRRotateAround (p,f), code
+  | TRMatrix p ->
+      let nx0,nc0 = num p.x0 in
+      let ny0,nc1 = num p.y0 in
+      let nxx,nc2 = num p.xx in
+      let nxy,nc3 = num p.xy in
+      let nyx,nc4 = num p.yx in
+      let nyy,nc5 = num p.yy in
+      let tname = Name.transform () in
+      C.TRName tname, 
+      nc0 ++ nc1 ++ nc2 ++ nc3 ++ nc4 ++ nc5 ++ 
+        C.CDefTrans (tname,{ C.x0 = nx0;
+                             y0 = ny0;
+                             xx = nxx;
+                             xy = nxy;
+                             yx = nyx;
+                             yy = nyy})
 
 and pen p = 
     match p.Hashcons.node with
@@ -350,8 +366,9 @@ and dash d =
   | DEvenly -> C.DEvenly, nop
   | DWithdots -> C.DWithdots, nop
   | DScaled (f, d) -> 
-      let d,c = dash d in
-        C.DScaled (f,d) , c
+      let f,c1 = num f in
+      let d,c2 = dash d in
+        C.DScaled (f,d) , c1 ++ c2
   | DShifted (p,d) ->
       let p, c1 = point p in
       let d, c2 = dash d in
@@ -372,8 +389,9 @@ and dash_pattern o =
 	
 and command c = 
     match c.Hashcons.node with 
-  | CDraw (p, color, pe, dsh) ->
+  | CDraw (p, b) ->
       let p, c1 = path p in
+      let {color = color; pen = pe; dash = dsh} = b.Hashcons.node in
       let pe, c2 = (option_compile pen) pe in
       let dsh, c3 = (option_compile dash) dsh in
       C.CSeq [c1; c2; c3; C.CDraw (p, color, pe, dsh)]
