@@ -14,14 +14,10 @@
 (*                                                                        *)
 (**************************************************************************)
 
-type scolor = 
-  | RGB of float * float * float
-  | CMYK of float * float * float * float
-  | Gray of float
+val add_set_verbosity : (bool -> unit) -> unit
+val set_verbosity : bool -> unit
 
-type color = 
-  |OPAQUE of scolor
-  |TRANSPARENT of float * scolor
+type color = Concrete_types.color
 
 type name = string
 
@@ -136,6 +132,10 @@ and path_node = private
 
 and path = path_node hash_consed
 
+and matrix = 
+    { xx : num; yx : num; 
+      xy : num; yy : num; x0 : num; y0 : num; }
+
 and transform_node = private
   | TRRotated of float
   | TRScaled of num
@@ -146,13 +146,14 @@ and transform_node = private
   | TRZscaled of point
   | TRReflect of point * point
   | TRRotateAround of point * float
+  | TRMatrix of matrix
 
 and transform = transform_node hash_consed
 
 and dash_node = private
   | DEvenly
   | DWithdots
-  | DScaled of float * dash
+  | DScaled of num * dash
   | DShifted of point * dash
   | DPattern of on_off list
 
@@ -174,7 +175,7 @@ and picture_node = private
 and picture = picture_node hash_consed
 
 and command_node = private
-  | CDraw of path * color option * pen option * dash option
+  | CDraw of path * brush
   | CFill of path * color option
   | CLabel of commandpic * position * point
   | CDotLabel of commandpic * position * point
@@ -197,6 +198,11 @@ and spec_image =
 
 and command = command_node hash_consed
 
+and brush_node = {pen : pen option;
+                  dash : dash option;
+                  color : color option}
+
+and brush = brush_node hash_consed
 (* smart constructors *)
 
 (* num *)
@@ -236,6 +242,8 @@ val mkTRShifted : point -> transform
 val mkTRSlanted : num -> transform
 val mkTRReflect : point -> point -> transform
 val mkTRRotateAround : point -> float -> transform
+val mkTRMatrix : matrix -> transform
+
 
 (* knot *)
 val mkKnot : direction -> point -> direction -> knot
@@ -289,7 +297,7 @@ val mkPIClip : commandpic -> path -> picture
 
 (* command *)
 
-val mkCDraw: path -> color option -> pen option -> dash option -> command
+val mkCDraw: path -> brush -> command
 val mkCFill: path -> color option -> command
 val mkCLabel: commandpic -> position -> point -> command
 val mkCDotLabel: commandpic -> position -> point -> command
@@ -304,7 +312,7 @@ val mkSeq : commandpic list -> commandpic
 
 val mkDEvenly: dash
 val mkDWithdots: dash
-val mkDScaled: float -> dash -> dash
+val mkDScaled: num -> dash -> dash
 val mkDShifted: point -> dash -> dash
 val mkDPattern: on_off list -> dash
 
@@ -315,6 +323,11 @@ val mkPenSquare: pen
 val mkPenFromPath: path -> pen
 val mkPenTransformed: pen -> transform -> pen
 
+(* brush *)
+val mkBrush: color option -> pen option -> dash option -> brush
+val mkBrushOpt : brush option -> color option -> pen option -> dash option 
+  -> brush
+
 (* on_off *)
 
 val mkOn : num -> on_off
@@ -330,3 +343,4 @@ val vreduce : vposition -> vposition_red
 val hreduce : hposition -> hposition_red
 
 val opposite_position : position -> position_red
+
